module RubygemsCheckReplacementVulnerability
  class Repository
    include ShellMethods

    attr_reader :repo_url, :work_dir

    def initialize(repo_url, work_dir)
      @repo_url = repo_url
      @work_dir = work_dir
    end

    def git_clone
      Dir.chdir(@work_dir) do
        run_command("git clone #{@repo_url} . --quiet")
      end
    end

    def tags
      return @tags if @tags

      tags = []

      Dir.chdir(@work_dir) do
        stdout = `git tag`

        stdout.each_line do |line|
          tags << line.strip
        end
      end

      @tags = tags
    end

    def find_version_tag(version)
      tags.find { |tag| tag == version || tag == "v#{version}" }
    end

    def checkout(hash)
      Dir.chdir(@work_dir) do
        run_command("git checkout #{hash} --quiet")
      end
    end

    def find_file(file, prefix = nil)
      array =
        if prefix
          [@work_dir, prefix, file]
        else
          [@work_dir, file]
        end

      Pathname.new(File.join(*array))
    end
  end
end
