module RubygemsCheckReplacementVulnerability
  require "open-uri"
  require "json"

  class Rubygems
    VULNERABLE_TERMS = Time.parse("2014-6-11")...Time.parse("2015-02-08")

    attr_reader :gem_name

    def initialize(gem_name)
      @gem_name = gem_name
    end

    def vulnerable_versions
      all_gems = self.class.get("https://rubygems.org/api/v1/versions/#{@gem_name}.json")
      vulnerable_gems = all_gems.select do |gem|
        created_at = Time.parse(gem["created_at"])
        VULNERABLE_TERMS.cover?(created_at)
      end

      vulnerable_gems.map{ |gem| gem["number"] }.sort_by { |version| Gem::Version.new(version) }
    end

    def gem_uri(version)
      "https://rubygems.org/gems/#{@gem_name}-#{version}.gem"
    end

    # @return [String] path to downloaded gem file
    def download_gem(version, dist_dir)
      gem_path = File.join(dist_dir, "#{@gem_name}-#{version}.gem")
      File.open(gem_path, "wb") do |f|
        f.write(open(gem_uri(version)).read)
      end
      gem_path
    end

    def self.owner_gems(user_id)
      gems = get("https://rubygems.org/api/v1/owners/#{user_id}/gems.json")
      gems.map { |gem| gem["name"] }
    end

    def self.get(url)
      JSON.parse(open(url).read)
    end
  end
end
